// devCtlVacScan.cc
extern "C" {
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
 
#include <semLib.h>
#include <tickLib.h>
#include <taskLib.h>
 
#include "dbAccess.h"
#include "dbDefs.h"
#include "link.h"
#include "epicsPrint.h"
#include "dbCommon.h"
#include "vacScanRecord.h"
#include "vacScanDefs.h"
#include "recSup.h"
}
 
#include "Message.h"
#include "Char8ArrayMessage.h"
#include "DevMpf.h"
 
static const int recBufSize = 1000;
static const int sendBufSize = 10;

#define MAX_CONSEC_ERRORS 2


class VsCtl : public DevMpf
{
public:
    VsCtl(dbCommon*,DBLINK*);

    long startIO(dbCommon* pr);
    long completeIO(dbCommon* pr,Message* m);
    static long dev_init(void*);
private:
    void cVarsSet(unsigned char buffer[20], double *var1, double *var2,
	unsigned short *var3, unsigned short *var4, double *var5, 
        unsigned short *var6, unsigned short *var7, unsigned short *var8,
	unsigned short *var9);
    int yCmdForm(int mode, int yfg, int ymfg, short var1, short var2,
	double var3, double var4, double var5, double var6, short var7,
	double var8, double var9);
    int zCmdForm(int mode, int zfg, int zmfg, int chan, short var1,
	short var2, double var3, double var4, short var5, double var6,
	double var7, short var8);
    unsigned char recBuf[recBufSize];
    unsigned char sendBuf[sendBufSize];
    int head;
};

MAKE_DSET(devVsCtlHideos,VsCtl::dev_init)

long VsCtl::dev_init(void* v)
{
    vacScanRecord* pr = (vacScanRecord*)v;
    VsCtl* pVsCtl = new VsCtl((dbCommon*)pr,&(pr->inp));
    return(pVsCtl->getStatus());
}

VsCtl::VsCtl(dbCommon* pr, link* l)
: DevMpf(pr,l,false), head(0)
{
    vmeio* io = (vmeio*)&(l->value);
    head=io->signal;
    if(!((head>=1)&&(head<=8))) {
	epicsPrintf("%s devVsCtl Illegal signal field\n",pr->name);
	pr->pact = true;
	return;
    }
}


int VsCtl::yCmdForm(int mode, int yfg, int ymfg, short var1, short var2,
    double var3, double var4, double var5, double var6, short var7,
    double var8, double var9)

{
    if ((ymfg == YMFG_LEAK)&&(mode != vacScanRMOD_Leak_Check)) return(-1);
    if ((ymfg == YMFG_BAR)&&(mode != vacScanRMOD_Bar_Chart)) return(-1);
    if ((ymfg == YMFG_PEAK)&&(mode != vacScanRMOD_User_Peaks)) return(-1);
    if ((ymfg == YMFG_TREND)&&(mode != vacScanRMOD_Trend)) return(-1);
    if ((ymfg == YMFG_ANALOG)&&(mode != vacScanRMOD_Analog)) return(-1);
    if(yfg & YFG_1) {
        if((mode == vacScanRMOD_Trend)||(mode == vacScanRMOD_User_Peaks))
            return(-1);
	sprintf((char *)sendBuf,"Y1,%d",var1); 
        return(0);
    }
    if(yfg & YFG_2) {
        if(mode == vacScanRMOD_Leak_Check) return(-1);
	sprintf((char *)sendBuf,"Y2,%d",var2); 
        return(0);
    }
    if(yfg & YFG_3) {
        var3 = var3 + 1;
        if(var3 < 1) var3 = 1;
        else if(var3 > 6) var3 = 6;
	sprintf((char *)sendBuf,"Y3,%d",(int)var3); 
        return(0);
    }
    if(yfg & YFG_4) {
        if((mode == vacScanRMOD_Trend)
	|| (mode == vacScanRMOD_User_Peaks)
	||(mode == vacScanRMOD_Leak_Check)) return(-1);
        if(var4 > 80) var4 = 80;
	sprintf((char *)sendBuf,"Y4,%d",(int)var4); 
        return(0);
    }
    if(yfg & YFG_5) {
        if(mode != vacScanRMOD_Bar_Chart) return(-1);
        if(var5 >= 100) var5 = 2;
	else if(var5 >=50) var5 = 1;
	else var5 = 0;
	sprintf((char *)sendBuf,"Y5,%d",(int)var5); 
        return(0);
    }
    if(yfg & YFG_6) {
        if((mode == vacScanRMOD_Trend)||(mode == vacScanRMOD_User_Peaks))
            return(-1);
	var6 *= -1;
        if(var6 > 14) var6 = 14;
	if(var6 < 8) var6 = 8;
	sprintf((char *)sendBuf,"Y6,%d",(int)var6); 
        return(0);
    }
    if(yfg & YFG_7) {
        if(mode != vacScanRMOD_Bar_Chart) return(-1);
	sprintf((char *)sendBuf,"Y7,%d",var7); 
        return(0);
    }
    if(yfg & YFG_8) {
        if(mode != vacScanRMOD_Trend) return(-1);
	if(var8 > 99) var8 = 99;
	if(var8 < 1) var8 = 1;
	sprintf((char *)sendBuf,"Y8,%d",(int)var8); 
        return(0);
    }
    if(yfg & YFG_9) {
        if(mode != vacScanRMOD_Leak_Check) return(-1);
	if(var9 > 100) var9 = 100;
	if(var9 < 1) var9 = 1;
	sprintf((char *)sendBuf,"Y9,%d",(int)var9); 
        return(0);
    }
    return(-1);
}

int VsCtl::zCmdForm(int mode, int zfg, int zmfg, int chan, short var1,
    short var2, double var3, double var4, short var5, double var6,
    double var7, short var8)

{
    if ((zmfg == ZMFG_PEAK)&&(mode != vacScanRMOD_User_Peaks)) return(-1);
    if ((zmfg == ZMFG_TREND)&&(mode != vacScanRMOD_Trend)) return(-1);
    if(zfg & ZFG_1) {
	sprintf((char *)sendBuf,"Z1,%d,%d",chan,var1);
        return(0);
    }
    if(zfg & ZFG_2) {
	sprintf((char *)sendBuf,"Z2,%d,%d",chan,var2);
        return(0);
    }
    if(zfg & ZFG_3) {
	if(var3 > 100) var3 = 100;
	if(var3 < 1) var3 = 1;
	sprintf((char *)sendBuf,"Z3,%d,%d",chan,(int)var3);
        return(0);
    }
    if(zfg & ZFG_4) {
	if(var4 > 11) var4 = 11;
	if(var4 < 5) var4 = 5;
	sprintf((char *)sendBuf,"Z4,%d,%d",chan,(int)var4);
        return(0);
    }
    if(zfg & ZFG_5) {
	sprintf((char *)sendBuf,"Z5,%d,%d",chan,var5);
        return(0);
    }
    if(zfg & ZFG_6) {
        if(mode != vacScanRMOD_User_Peaks) return(-1);
	var6 *= 10;
	if(var6 > 99) var6 = 99;
	if(var6 < 1) var6 = 1;
	sprintf((char *)sendBuf,"Z6,%d,%d",chan,(int)var6);
        return(0);
    }
    if(zfg & ZFG_7) {
        if(mode != vacScanRMOD_User_Peaks) return(-1);
	sprintf((char *)sendBuf,"Z7,%d,%d",chan,(int)var7);
        return(0);
    }
    if(zfg & ZFG_8) {
        if(mode != vacScanRMOD_User_Peaks) return(-1);
	sprintf((char *)sendBuf,"Z8,%d,%d",chan,var8);
        return(0);
    }
    return(-1);
}

long VsCtl::startIO(dbCommon* pr) 
{
    int sendBail = -1;
    int holdInt;
    int holdMode = 6;
    cmdType type;

    vacScanRecord* vsCtl = (vacScanRecord*)pr;
    for(holdInt = 0; holdInt < 10; holdInt++) sendBuf[holdInt] = 0;
    /* We have to check to see if any record fields changed */
    if (vsCtl->flgs) {
        sendBail = 0;
        if (vsCtl->flgs & VS_FLGS_CMOD)
        {
            sendBuf[0] = 'M';
            switch (vsCtl->cmod) {
                case 0:
                case 1:
                case 2:
                    holdMode = vsCtl->cmod;
                    break;
                case 3:
                case 4:
                case 5:
                    holdMode = vsCtl->cmod + 2;
                    break;
                case 6:
                    holdMode = vsCtl->cmod + 9;
                    break;
                case 7:
                    holdMode =  vsCtl->cmod + 12;
                    break;
                default:
                    holdMode = 6;
                    break;
            }
	    sprintf((char *)sendBuf,"M%d",holdMode);
        } else if (vsCtl->flgs & VS_FLGS_CLOC) {
	    sprintf((char *)sendBuf,"L%d",vsCtl->cloc);
        } else if (vsCtl->flgs & VS_FLGS_CXF1) {
	    sprintf((char *)sendBuf,"X1,%d",vsCtl->cxf1);
        } else if (vsCtl->flgs & VS_FLGS_CXF2) {
	    sprintf((char *)sendBuf,"X2,%d",vsCtl->cxf2);
        } else if (vsCtl->flgs & VS_FLGS_CRST) {
	    sprintf((char *)sendBuf,"R");
        }
    } else if ((vsCtl->ymfg)&&(vsCtl->yfg)) {
        sendBail = 0;
        switch (vsCtl->ymfg) {
        case YMFG_BAR:
            sendBail = yCmdForm(vsCtl->rmod, vsCtl->yfg, vsCtl->ymfg,
		vsCtl->cybm, vsCtl->cyba, vsCtl->cybs, vsCtl->cybp,
                vsCtl->cybt, vsCtl->cybg, vsCtl->cybh, NULL, NULL);
            break;
        case YMFG_LEAK:
            sendBail = yCmdForm(vsCtl->rmod, vsCtl->yfg, vsCtl->ymfg,
		vsCtl->cylm, NULL, vsCtl->cyls, NULL, NULL, vsCtl->cylg,
		NULL, NULL, vsCtl->cyli);
            break;
        case YMFG_PEAK:
            sendBail = yCmdForm(vsCtl->rmod, vsCtl->yfg, vsCtl->ymfg,
		NULL, vsCtl->cypa, vsCtl->cyps, NULL,
                NULL, NULL, NULL, NULL, NULL);
            break;
        case YMFG_TREND:
            sendBail = yCmdForm(vsCtl->rmod, vsCtl->yfg, vsCtl->ymfg,
		NULL, vsCtl->cyta, vsCtl->cyts, NULL,
                NULL, NULL, NULL, vsCtl->cytn, NULL);
            break;
       case YMFG_ANALOG:

            sendBail = yCmdForm(vsCtl->rmod, vsCtl->yfg, vsCtl->ymfg,
		vsCtl->cyam, vsCtl->cyaa, vsCtl->cyas, vsCtl->cyap,
                NULL, vsCtl->cyag, NULL, NULL, NULL);
            break;
       default:
            sendBail = -1;
            break;
        }
    } else if ((vsCtl->zmfg)&&(vsCtl->zcfg)&&(vsCtl->zfg)) {
        sendBail = 0;
        switch (vsCtl->zmfg)
        {
        case ZMFG_PEAK:
            switch (vsCtl->zcfg) {
            case ZCFG_1:
                sendBail = zCmdForm(vsCtl->rmod, vsCtl->zfg, vsCtl->zmfg,
		    1, vsCtl->c1pm, vsCtl->c1pc, vsCtl->c1ps, vsCtl->c1pg,
                    vsCtl->c1pa, vsCtl->c1pl, vsCtl->c1pn, vsCtl->c1pk);
                break;
            case ZCFG_2:
                sendBail = zCmdForm(vsCtl->rmod, vsCtl->zfg, vsCtl->zmfg,
		    2, vsCtl->c2pm, vsCtl->c2pc, vsCtl->c2ps, vsCtl->c2pg,
                    vsCtl->c2pa, vsCtl->c2pl, vsCtl->c2pn, vsCtl->c2pk);
                break;
            case ZCFG_3:
                sendBail = zCmdForm(vsCtl->rmod, vsCtl->zfg, vsCtl->zmfg,
		    3, vsCtl->c3pm, vsCtl->c3pc, vsCtl->c3ps, vsCtl->c3pg,
                    vsCtl->c3pa, vsCtl->c3pl, vsCtl->c3pn, vsCtl->c3pk);
                break;
            case ZCFG_4:
                sendBail = zCmdForm(vsCtl->rmod, vsCtl->zfg, vsCtl->zmfg,
		    4, vsCtl->c4pm, vsCtl->c4pc, vsCtl->c4ps, vsCtl->c4pg,
                    vsCtl->c4pa, vsCtl->c4pl, vsCtl->c4pn, vsCtl->c4pk);
                break;
            case ZCFG_5:
                sendBail = zCmdForm(vsCtl->rmod, vsCtl->zfg, vsCtl->zmfg,
		    5, vsCtl->c5pm, vsCtl->c5pc, vsCtl->c5ps, vsCtl->c5pg,
                    vsCtl->c5pa, vsCtl->c5pl, vsCtl->c5pn, vsCtl->c5pk);
                break;
            case ZCFG_6:
                sendBail = zCmdForm(vsCtl->rmod, vsCtl->zfg, vsCtl->zmfg,
		    6, vsCtl->c6pm, vsCtl->c6pc, vsCtl->c6ps, vsCtl->c6pg,
                    vsCtl->c6pa, vsCtl->c6pl, vsCtl->c6pn, vsCtl->c6pk);
                break;
            default:
                sendBail = -1;
                break;
            }
            break;
        case ZMFG_TREND:
            switch (vsCtl->zcfg) {
            case ZCFG_1:
                sendBail = zCmdForm(vsCtl->rmod, vsCtl->zfg, vsCtl->zmfg,
		    1, vsCtl->c1tm, vsCtl->c1tc, vsCtl->c1ts, vsCtl->c1tg,
                    vsCtl->c1ta, NULL, NULL, NULL);
                break;
            case ZCFG_2:
                sendBail = zCmdForm(vsCtl->rmod, vsCtl->zfg, vsCtl->zmfg,
		    2, vsCtl->c2tm, vsCtl->c2tc, vsCtl->c2ts, vsCtl->c2tg,
                    vsCtl->c2ta, NULL, NULL, NULL);
                break;
            case ZCFG_3:
                sendBail = zCmdForm(vsCtl->rmod, vsCtl->zfg, vsCtl->zmfg,
		    3, vsCtl->c3tm, vsCtl->c3tc, vsCtl->c3ts, vsCtl->c3tg,
                    vsCtl->c3ta, NULL, NULL, NULL);
                break;
            case ZCFG_4:
                sendBail = zCmdForm(vsCtl->rmod, vsCtl->zfg, vsCtl->zmfg,
		    4, vsCtl->c4tm, vsCtl->c4tc, vsCtl->c4ts, vsCtl->c4tg,
                    vsCtl->c4ta, NULL, NULL, NULL);
                break;
            case ZCFG_5:
                sendBail = zCmdForm(vsCtl->rmod, vsCtl->zfg, vsCtl->zmfg,
		    5, vsCtl->c5tm, vsCtl->c5tc, vsCtl->c5ts, vsCtl->c5tg,
                    vsCtl->c5ta, NULL, NULL, NULL);
                break;
            case ZCFG_6:
                sendBail = zCmdForm(vsCtl->rmod, vsCtl->zfg, vsCtl->zmfg,
		    6, vsCtl->c6tm, vsCtl->c6tc, vsCtl->c6ts, vsCtl->c6tg,
                    vsCtl->c6ta, NULL, NULL, NULL);
                break;
           default:
                sendBail = -1;
                break;
           }
           break;
       default:
           sendBail = -1;
           break;
       }

    } else {
        sendBail = -1;
    }
    vsCtl->flgs = 0;
    vsCtl->ymfg = 0;
    vsCtl->yfg = 0;
    vsCtl->zmfg = 0;
    vsCtl->zfg = 0;
    vsCtl->zcfg = 0;
    if(sendBail == -1) {
        strcpy((char *)sendBuf, "0");
	type = cmdGetCtl;
    } else {
	type = cmdPutGetCtl;
    }
    Char8ArrayMessage* message = new Char8ArrayMessage;
    int lenSend = ::strlen((char *)sendBuf) + 1;
    message->allocValue(lenSend);
    ::memcpy(message->value,sendBuf,lenSend);
    message->setSize(lenSend);
    message->cmd = type;
    int status = sendReply(message);
    return(status);
}

void VsCtl::cVarsSet(unsigned char buffer[20], double *var1, double *var2,
    unsigned short *var3, unsigned short *var4, double *var5,
    unsigned short *var6, unsigned short *var7, unsigned short *var8,
    unsigned short *var9)
{
    int holdInt;
    unsigned char holdChar;
    float holdFloat;


    if(buffer[0] == 255) {
        *var9 = 0;
        *var1 = 0;
    } else {
        *var9 = 1;
        *var1 = buffer[0];
    }

    holdFloat = buffer[1];
    *var2 = holdFloat/10;

    holdChar = (buffer[2] & 0xFF);

    *var3 = 1 && (holdChar & 0x80); 
    *var4 = 1 && (holdChar & 0x40); 

    holdInt = -1 * (holdChar & 0x0F);
    *var5 = holdInt;

    holdChar = (buffer[3] & 0x0F);

    *var6 = 1 && (holdChar & 0x04); 
    *var7 = 1 && (holdChar & 0x02); 
    *var8 = 1 && (holdChar & 0x01); 
}

long VsCtl::completeIO(dbCommon* pr,Message* m)
{
    vacScanRecord* vsCtl = (vacScanRecord*)pr;
    if(m->getType() != messageTypeChar8Array) {
        if(m->getType() != messageTypeInt32)
            epicsPrintf("VsCtl::completeIO illegal message. %s\n",
                pr->name);
        (vsCtl->val)++;
        if ((vsCtl->val) >= MAX_CONSEC_ERRORS)
            recGblSetSevr(pr,READ_ALARM,INVALID_ALARM);
        delete m;
        return(0);
    }
    Char8ArrayMessage *message = (Char8ArrayMessage *)m;
    int32 rtnSize = message->getSize();
    if(rtnSize>=recBufSize) {
        epicsPrintf("VsCtl::completeIO message too big\n");
        recGblSetSevr(pr,READ_ALARM,INVALID_ALARM);
        delete m;
        return(0);
    }
    ::memcpy(recBuf,message->value,rtnSize);
    int modeInt;
    int holdInt;
    unsigned char holdChar;

    vsCtl->udf=0;

    modeInt = recBuf[0];
    vsCtl->rdrs = 0; 
    vsCtl->rdup = 0; 
    vsCtl->rdbc = 0; 
    vsCtl->rdlc = 0; 
    vsCtl->rdmm = 0; 
    vsCtl->rdan = 0; 
    vsCtl->rdvs = 0; 
    vsCtl->rdut = 0; 
    vsCtl->rddt = 0; 
    vsCtl->rdal = 0; 
    vsCtl->rddg = 0; 
    vsCtl->rdmc = 0; 
    vsCtl->rddi = 0; 
    vsCtl->rdtr = 0; 
    switch(modeInt) {
    case 0:
        vsCtl->rmod = vacScanRMOD_RGA_Select; vsCtl->rdrs = 1; break;
    case 1:
        vsCtl->rmod = vacScanRMOD_User_Peaks; vsCtl->rdup = 1; break;
    case 2:
        vsCtl->rmod = vacScanRMOD_Bar_Chart; vsCtl->rdbc = 1; break;
    case 5:
        vsCtl->rmod = vacScanRMOD_Leak_Check; vsCtl->rdlc = 1; break;
    case 6:
        vsCtl->rmod = vacScanRMOD_Main_Menu; vsCtl->rdmm = 1; break;
    case 7:
        vsCtl->rmod = vacScanRMOD_Analog; vsCtl->rdan = 1; break;
    case 8:
    case 9:
    case 10:
        vsCtl->rmod = vacScanRMOD_Vacuum_Scan; vsCtl->rdvs = 1; break;
    case 11:
        vsCtl->rmod = vacScanRMOD_Utilities; vsCtl->rdut = 1; break;
    case 12:
        vsCtl->rmod = vacScanRMOD_Date_Time; vsCtl->rddt = 1; break;
    case 13:
        vsCtl->rmod = vacScanRMOD_Alarm; vsCtl->rdal = 1; break;
    case 15:
        vsCtl->rmod = vacScanRMOD_Degas; vsCtl->rddg = 1; break;
    case 17:
        vsCtl->rmod = vacScanRMOD_Multiplier_Calibrate; vsCtl->rdmc = 1; break;
    case 18:
        vsCtl->rmod = vacScanRMOD_Diagnostics; vsCtl->rddi = 1; break;
    case 19:
        vsCtl->rmod = vacScanRMOD_Trend; vsCtl->rdtr = 1; break;
    default:
        vsCtl->rmod = vacScanRMOD_Undefined;
        break;
    }

    holdInt = recBuf[1] - 1;
    vsCtl->rbcf = holdInt;

    holdInt = recBuf[2];
    vsCtl->rbcs = holdInt;

    holdInt = recBuf[3];
    vsCtl->rafm = holdInt;

    holdInt = recBuf[4];
    vsCtl->rtri = holdInt;

    holdInt = recBuf[9];
    vsCtl->rprg = holdInt;

    holdChar = recBuf[5];

    holdInt = 1 && (holdChar & 0x80);
    switch(holdInt)
    {
    case 0:
        vsCtl->rdet = vacScanRDET_Faraday;
        break;
    case 1:
        vsCtl->rdet = vacScanRDET_Multiplier;
        break;
    default:
        vsCtl->rdet = vacScanRDET_Faraday;
        break;
    }

    holdInt = 1 && (holdChar & 0x40);
    switch(holdInt)
    {
    case 0:
        vsCtl->ragn = vacScanCXF1_Off;
        break;
    case 1:
        vsCtl->ragn = vacScanCXF1_On;
        break;
    default:
        vsCtl->ragn = vacScanCXF1_Off;
        break;
    }

    holdInt = 1 && (holdChar & 0x10);
    switch(holdInt)
    {
    case 0:
        vsCtl->rtpr = vacScanCXF1_Off;
        break;
    case 1:
        vsCtl->rtpr = vacScanCXF1_On;
        break;
    default:
        vsCtl->rtpr = vacScanCXF1_Off;
        break;
    }

    holdInt = -1 * ((holdChar & 0x0F));
    vsCtl->rgnr = holdInt;

    holdChar = recBuf[6];

    holdInt = 0x07 & ((holdChar & 0x70) >> 4);
    holdInt = holdInt - 1;
    vsCtl->rssp = holdInt;

    holdInt = 1 && (holdChar & 0x08);
    switch(holdInt)
    {
    case 0:
        vsCtl->rfs2 = vacScanRFS1_OK;
        break;
    case 1:
        vsCtl->rfs2 = vacScanRFS1_Failed;
        break;
    default:
        vsCtl->rfs2 = vacScanRFS1_Failed;
        break;
    }

    holdInt = 1 && (holdChar & 0x04);
    switch(holdInt)
    {
    case 0:
        vsCtl->rfs1 = vacScanRFS1_OK;
        break;
    case 1:
        vsCtl->rfs1 = vacScanRFS1_Failed;
        break;
    default:
        vsCtl->rfs1 = vacScanRFS1_Failed;
        break;
    }

    holdInt = 1 && (holdChar & 0x02);
    switch(holdInt)
    {
    case 0:
        vsCtl->rfsl = vacScanRFSL_Filament_1;
        break;
    case 1:
        vsCtl->rfsl = vacScanRFSL_Filament_2;
        break;
    default:
        vsCtl->rfsl = vacScanRFSL_Filament_1;
        break;
    }

    holdInt = 1 && (holdChar & 0x01);
    switch(holdInt)
    {
    case 0:
        vsCtl->rfon = vacScanCXF1_Off;
        break;
    case 1:
        vsCtl->rfon = vacScanCXF1_On;
        break;
    default:
        vsCtl->rfon = vacScanCXF1_Off;
        break;
    }

    switch(modeInt) {
    case 1:
        cVarsSet(&recBuf[10], &vsCtl->r1pm, &vsCtl->r1pv, &vsCtl->r1pd,
		&vsCtl->r1pa, &vsCtl->r1pg, &vsCtl->r1pi, &vsCtl->r1ph,
		&vsCtl->r1pl, &vsCtl->r1pc);
        cVarsSet(&recBuf[20], &vsCtl->r2pm, &vsCtl->r2pv, &vsCtl->r2pd,
		&vsCtl->r2pa, &vsCtl->r2pg, &vsCtl->r2pi, &vsCtl->r2ph,
		&vsCtl->r2pl, &vsCtl->r2pc);
        cVarsSet(&recBuf[30], &vsCtl->r3pm, &vsCtl->r3pv, &vsCtl->r3pd,
		&vsCtl->r3pa, &vsCtl->r3pg, &vsCtl->r3pi, &vsCtl->r3ph,
		&vsCtl->r3pl, &vsCtl->r3pc);
        cVarsSet(&recBuf[40], &vsCtl->r4pm, &vsCtl->r4pv, &vsCtl->r4pd,
		&vsCtl->r4pa, &vsCtl->r4pg, &vsCtl->r4pi, &vsCtl->r4ph,
		&vsCtl->r4pl, &vsCtl->r4pc);
        cVarsSet(&recBuf[50], &vsCtl->r5pm, &vsCtl->r5pv, &vsCtl->r5pd,
		&vsCtl->r5pa, &vsCtl->r5pg, &vsCtl->r5pi, &vsCtl->r5ph,
		&vsCtl->r5pl, &vsCtl->r5pc);
        cVarsSet(&recBuf[60], &vsCtl->r6pm, &vsCtl->r6pv, &vsCtl->r6pd,
		&vsCtl->r6pa, &vsCtl->r6pg, &vsCtl->r6pi, &vsCtl->r6ph,
		&vsCtl->r6pl, &vsCtl->r6pc);
        break;

    case 19:
        cVarsSet(&recBuf[10], &vsCtl->r1tm, &vsCtl->r1tv, &vsCtl->r1td,
		&vsCtl->r1ta, &vsCtl->r1tg, &vsCtl->r1ti, &vsCtl->r1th,
		&vsCtl->r1tl, &vsCtl->r1tc);
        cVarsSet(&recBuf[20], &vsCtl->r2tm, &vsCtl->r2tv, &vsCtl->r2td,
		&vsCtl->r2ta, &vsCtl->r2tg, &vsCtl->r2ti, &vsCtl->r2th,
		&vsCtl->r2tl, &vsCtl->r2tc);
        cVarsSet(&recBuf[30], &vsCtl->r3tm, &vsCtl->r3tv, &vsCtl->r3td,
		&vsCtl->r3ta, &vsCtl->r3tg, &vsCtl->r3ti, &vsCtl->r3th,
		&vsCtl->r3tl, &vsCtl->r3tc);
        cVarsSet(&recBuf[40], &vsCtl->r4tm, &vsCtl->r4tv, &vsCtl->r4td,
		&vsCtl->r4ta, &vsCtl->r4tg, &vsCtl->r4ti, &vsCtl->r4th,
		&vsCtl->r4tl, &vsCtl->r4tc);
        cVarsSet(&recBuf[50], &vsCtl->r5tm, &vsCtl->r5tv, &vsCtl->r5td,
		&vsCtl->r5ta, &vsCtl->r5tg, &vsCtl->r5ti, &vsCtl->r5th,
		&vsCtl->r5tl, &vsCtl->r5tc);
        cVarsSet(&recBuf[60], &vsCtl->r6tm, &vsCtl->r6tv, &vsCtl->r6td,
		&vsCtl->r6ta, &vsCtl->r6tg, &vsCtl->r6ti, &vsCtl->r6th,
		&vsCtl->r6tl, &vsCtl->r6tc);
        break;

    default:
        break;
    }

    holdInt = (int)(recBuf[70]);
    vsCtl->nelm = holdInt;
    if(holdInt>0) {
        vsCtl->wfsc = 0;
    } else {
        vsCtl->wfsc = 1;
    }

    for(holdInt = 90; holdInt<=150; holdInt += 10) {
	char *sp = (char *)&recBuf[holdInt];
	if(::strncmp(sp,"-.-",3)==0) {
	    ::memcpy(sp,"0.0",3);
	} else if(::strncmp(sp,">>>",3)==0) {
	    ::memcpy(sp,"9.9",3);
	}
    }
    vsCtl->rc0v = strtod((char*)&recBuf[90], NULL);
    vsCtl->rc1v = strtod((char*)&recBuf[100], NULL);
    vsCtl->rc2v = strtod((char*)&recBuf[110], NULL);
    vsCtl->rc3v = strtod((char*)&recBuf[120], NULL);
    vsCtl->rc4v = strtod((char*)&recBuf[130], NULL);
    vsCtl->rc5v = strtod((char*)&recBuf[140], NULL);
    vsCtl->rc6v = strtod((char*)&recBuf[150], NULL);

    vsCtl->val = 0;

    vsCtl->udf = FALSE;
    delete m;
    return(0);
}

