// devWfVacScan.cc
extern "C" {
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
 
#include <semLib.h>
#include <tickLib.h>
#include <taskLib.h>
 
#include "dbAccess.h"
#include "dbDefs.h"
#include "link.h"
#include "epicsPrint.h"
#include "dbCommon.h"
#include "waveformRecord.h"
#include "recSup.h"
}

#include "Message.h"
#include "Char8ArrayMessage.h"
#include "DevMpf.h"
#include "vacScanDefs.h"

int devWfVacScanDebug = 0;
static const int recBufSize = 1000;
class VsWf : public DevMpf
{
public:
    VsWf(dbCommon*,DBLINK*);

    long startIO(dbCommon* pr);
    long completeIO(dbCommon* pr,Message* m);
    static long dev_init(void*);
private:
    char recBuf[recBufSize];
    cmdType type;
    int axis;
    // disallow
    VsWf();
    VsWf(const VsWf &);
    VsWf& operator=(const VsWf&);
};

MAKE_DSET(devVsWfHideos,VsWf::dev_init)

long VsWf::dev_init(void* v)
{
    waveformRecord* pr = (waveformRecord*)v;
    VsWf* pVsWf = new VsWf((dbCommon*)pr,&(pr->inp));
    return(pVsWf->getStatus());
}

VsWf::VsWf(dbCommon* pr,link* l)
: DevMpf(pr,l,false), axis(0)
{
    vmeio* io = (vmeio*)&(l->value);
    waveformRecord* wf = (waveformRecord*)pr;
    if((wf->ftvl!=DBF_FLOAT) && (wf->ftvl!=DBF_DOUBLE)) {
        epicsPrintf("%s devVsWfHideos (init_record) Illegal data type\n",
            pr->name);
        pr->pact = true;
	return;
    }
    axis=io->signal;
    if(axis==1) {
        type = cmdGetPeak;
printf("%s type is cmdGetPeak %d\n",wf->name,type);
    } else if(axis==2) {
        type = cmdGetX;
printf("%s type is cmdGetX %d\n",wf->name,type);
    } else {
        epicsPrintf("%s devVsWfHideos (init_record) Illegal signal field\n",
            pr->name);
        pr->pact = true;
    }
}

long VsWf::startIO(dbCommon*)
{
    Char8ArrayMessage* message = new Char8ArrayMessage;
    int lenSend = 1;
    message->allocValue(lenSend);
    message->value[0] = 0;
    message->setSize(lenSend);
    message->cmd = type;
    int status = sendReply(message);
    return(status);
}

long VsWf::completeIO(dbCommon* pr,Message* m)
{
    waveformRecord* wf_rec = (waveformRecord*)pr;
    if(m->getType() != messageTypeChar8Array) {
	if(m->getType() != messageTypeInt32)
	    epicsPrintf("VsWf::completeIO illegal message. %s\n",
		wf_rec->name);
	recGblSetSevr(pr,READ_ALARM,INVALID_ALARM);
	delete m;
	return(0);
    }
    Char8ArrayMessage *message = (Char8ArrayMessage *)m;
    int32 rtnSize = message->getSize();
    if(rtnSize>=recBufSize) {
	epicsPrintf("VsWf::completeIO message too big\n");
	recGblSetSevr(pr,READ_ALARM,INVALID_ALARM);
	delete m;
	return(0);
    }
    ::memcpy(recBuf,message->value,rtnSize);
    recBuf[rtnSize] = 0;
    if(devWfVacScanDebug)
	printf("%s type %d received |%s|\n",wf_rec->name,type,recBuf);
    uint32 i=0;
    char *sp;
    char *save;
    switch(wf_rec->ftvl) {
    case DBF_FLOAT: {
            float *f;
            f=(float*)(wf_rec->bptr);
            for(sp=strtok_r(recBuf,",",&save);sp;sp=strtok_r(NULL,",",&save)) {
                if(i>=wf_rec->nelm) break;
		if(::strncmp(sp,".-.",3)==0) {
		    ::memcpy(sp,"0.0",3);
		} else if(::strncmp(sp,">>>",3)==0) {
		    ::memcpy(sp,"9.9",3);
		}
                sscanf(sp,"%f",&f[i++]);
            }
	    for(unsigned int j=i; j<wf_rec->nelm; j++) f[j] = 0.0;
	}
        break;
    case DBF_DOUBLE: {
            double *d;
            d=(double*)(wf_rec->bptr);
            for(sp=strtok_r(recBuf,",",&save);sp;sp=strtok_r(NULL,",",&save)) {
                if(i>=wf_rec->nelm) break;
		if(::strncmp(sp,"-.-",3)==0) {
		    ::memcpy(sp,"0.0",3);
		} else if(::strncmp(sp,">>>",3)==0) {
		    ::memcpy(sp,"9.9",3);
		}
                sscanf(sp,"%lf",&d[i++]);
            }
	    for(unsigned int j=i; j<wf_rec->nelm; j++) d[j] = 0.0;
	}
        break;
    default:
        break;
    }
    wf_rec->nord=i;
    wf_rec->udf=0;
    delete m;
    return(0);
}
